#ifndef __GURUMDDS_TYPESUPPORT_H__
#define __GURUMDDS_TYPESUPPORT_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <inttypes.h>
#include <stdbool.h>
#include <stdlib.h>

/**
 * @file
 * GurumDDS TypeSupport APIs
 */

/** TypeSupport interface is an abstract interface that has to be specialized for each concrete type that will be used by the application */
typedef struct _dds_TypeSupport dds_TypeSupport;

/**
 * Create TypeSupport
 *
 * @param meta_str CDR Metadata string of Type
 *
 * @return newly created typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_TypeSupport_create(const char* meta_str);

/**
 * Create TypeSupport
 *
 * @param meta_str_array CDR Metadata string array of type
 * @param meta_str_array_len size of string array
 *
 * @return newly created typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_TypeSupport_create2(const char** meta_str_array, uint32_t meta_str_array_len);

/**
 * Create uninitialized TypeSupport
 *
 * @param meta_str CDR Metadata string of Type
 *
 * @return newly created typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_TypeSupport_early_create(const char* meta_str);

/**
 * Create uninitialized TypeSupport
 *
 * @param meta_str_array CDR Metadata string array of type
 * @param meta_str_array_len size of string array
 *
 * @return newly created typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_TypeSupport_early_create2(const char** meta_str_array, uint32_t meta_str_array_len);

/**
 * Refer to other type
 *
 * @param self early typesupport
 * @param other other typesupport
 *
 * @return standard dds return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeSupport_early_refer_to_type(dds_TypeSupport* self, dds_TypeSupport* other);

/**
 * Initialize TypeSupport
 *
 * @param self early typesupport
 *
 * @return standard dds return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeSupport_early_initialize(dds_TypeSupport* self);

/**
 * Free TypeSupport
 *
 * @param self typesupport
 */
GURUMDDS_C_DLLAPI void dds_TypeSupport_delete(dds_TypeSupport* self);

/**
 * Register Topic Type
 *
 * @param self typesupport
 * @param participant domain participant
 * @param type_name name of topic type
 *
 * @return standard dds return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeSupport_register_type(dds_TypeSupport* self, dds_DomainParticipant* participant, const char* type_name);

/**
 * Get default name of Topic Type
 *
 * @param self typesupport
 *
 * @return default type name
 */
GURUMDDS_C_DLLAPI const char* dds_TypeSupport_get_type_name(dds_TypeSupport* self);

/**
 * Allocate new UserData
 *
 * @param self typesupport
 *
 * @return newly allocated userdata object
 */
GURUMDDS_C_DLLAPI void* dds_TypeSupport_alloc(dds_TypeSupport* self);

/**
 * Free UserData
 *
 * @param self typesupport
 * @param data userdata object to be free'd
 */
GURUMDDS_C_DLLAPI void dds_TypeSupport_free(dds_TypeSupport* self, void* data);

/**
 * Get CDR metadata of Topic Type
 *
 * @param self typesupport
 *
 * @return CDR metadata
 */
GURUMDDS_C_DLLAPI void* dds_TypeSupport_get_cdr_meta(dds_TypeSupport* self);

/**
 * Clone UserData
 *
 * @param self typesupport
 * @param data userdata pointer
 *
 * @return cloned userdata pointer
 */
GURUMDDS_C_DLLAPI void* dds_TypeSupport_clone_data(dds_TypeSupport* self, void* data);

/**
 * Serialize UserData
 *
 * @param self typesupport
 * @param data userdata pointer
 * @param output_size size of serialized userdata
 *
 * @return newly allocated userdata octets
 */
GURUMDDS_C_DLLAPI void* dds_TypeSupport_serialize(dds_TypeSupport* self, void* data, size_t* output_size);

/**
 * Serialize UserData with given data representation id
 *
 * @param self typesupport
 * @param data userdata pointer
 * @param output_size size of serialized userdata
 * @param repr_id data representation id
 *
 * @return newly allocated userdata octets
 */
GURUMDDS_C_DLLAPI void* dds_TypeSupport_serialize_w_repr_id(dds_TypeSupport* self, void* data, size_t* output_size, dds_DataRepresentationId_t repr_id);

/**
 * Deserialize UserData
 *
 * @param self typesupport
 * @param input userdata octets
 * @param input_size size of userdata octets
 *
 * @return newly allocated userdata object
 */
GURUMDDS_C_DLLAPI void* dds_TypeSupport_deserialize(dds_TypeSupport* self, void* input, size_t input_size);

// IDL Primitive Sequence Supports
// dds_Char Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Char sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Char sequence
 */
GURUMDDS_C_DLLAPI dds_CharSeq* dds_CharSeq_create(uint32_t capacity);

/**
 * Clone dds_Char sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Char sequence
 */
GURUMDDS_C_DLLAPI dds_CharSeq* dds_CharSeq_clone(dds_CharSeq* sequence);

/**
 * Delete dds_Char sequence
 *
 * @param dds_Char sequence
 */
GURUMDDS_C_DLLAPI void dds_CharSeq_delete(dds_CharSeq* sequence);

/**
 * Get length of dds_Char sequence
 *
 * @param dds_Char sequence
 *
 * @return length of dds_Char sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_CharSeq_length(dds_CharSeq* sequence);

/**
 * Get capacity of dds_Char sequence
 *
 * @param dds_Char sequence
 *
 * @return capacity of dds_Char sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_CharSeq_capacity(dds_CharSeq* sequence);

/**
 * Add an item to dds_Char sequence
 *
 * @param dds_Char sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_CharSeq_add(dds_CharSeq* sequence, dds_Char item);

/**
 * Add an item to dds_Char sequence at given position
 *
 * @param dds_Char sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_CharSeq_add_at(dds_CharSeq* sequence, uint32_t index, dds_Char item);

/**
 * Remove an item from dds_Char sequence
 *
 * @param dds_Char sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Char dds_CharSeq_remove(dds_CharSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Char sequence
 *
 * @param dds_Char sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Char dds_CharSeq_get(dds_CharSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Char sequence
 *
 * @param dds_Char sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_CharSeq_add_array(dds_CharSeq* sequence, dds_Char* array, uint32_t size);

/**
 * Get an array of items from dds_Char sequence
 *
 * @param dds_Char sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_CharSeq_get_array(dds_CharSeq* sequence, dds_Char* array, uint32_t offset, uint32_t size);

// dds_String Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_String sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_String sequence
 */
GURUMDDS_C_DLLAPI dds_StringSeq* dds_StringSeq_create(uint32_t capacity);

/**
 * Copy dds_String sequence from source to sequence
 *
 * @param sequence destination sequence
 * @param source source sequence
 *
 * @return size for copied dds_String sequence
 */
GURUMDDS_C_DLLAPI size_t dds_StringSeq_copy(dds_StringSeq* sequence, dds_StringSeq* source);

/**
 * Clone dds_String sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_String sequence
 */
GURUMDDS_C_DLLAPI dds_StringSeq* dds_StringSeq_clone(dds_StringSeq* sequence);

/**
 * Delete dds_String sequence
 *
 * @param dds_String sequence
 */
GURUMDDS_C_DLLAPI void dds_StringSeq_delete(dds_StringSeq* sequence);

/**
 * Delete dds_String sequence
 *
 * @param dds_String sequence
 */
GURUMDDS_C_DLLAPI void dds_StringSeq_delayed_delete(dds_StringSeq* sequence);

/**
 * Get length of dds_String sequence
 *
 * @param dds_String sequence
 *
 * @return length of dds_String sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_StringSeq_length(dds_StringSeq* sequence);

/**
 * Get capacity of dds_String sequence
 *
 * @param dds_String sequence
 *
 * @return capacity of dds_String sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_StringSeq_capacity(dds_StringSeq* sequence);

/**
 * Add an item to dds_String sequence
 *
 * @param dds_String sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_StringSeq_add(dds_StringSeq* sequence, dds_String item);

/**
 * Add an item to dds_String sequence at given position
 *
 * @param dds_String sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_StringSeq_add_at(dds_StringSeq* sequence, uint32_t index, dds_String item);

/**
 * Remove an item from dds_String sequence
 *
 * @param dds_String sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_String dds_StringSeq_remove(dds_StringSeq* sequence, uint32_t index);

/**
 * Get an item from dds_String sequence
 *
 * @param dds_String sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_String dds_StringSeq_get(dds_StringSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_String sequence
 *
 * @param dds_String sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_StringSeq_add_array(dds_StringSeq* sequence, dds_String* array, uint32_t size);

/**
 * Get an array of items from dds_String sequence
 *
 * @param dds_String sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_StringSeq_get_array(dds_StringSeq* sequence, dds_String* array, uint32_t offset, uint32_t size);

// dds_Wchar Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Wchar sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Wchar sequence
 */
GURUMDDS_C_DLLAPI dds_WcharSeq* dds_WcharSeq_create(uint32_t capacity);

/**
 * Clone dds_Wchar sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Wchar sequence
 */
GURUMDDS_C_DLLAPI dds_WcharSeq* dds_WcharSeq_clone(dds_WcharSeq* sequence);

/**
 * Delete dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 */
GURUMDDS_C_DLLAPI void dds_WcharSeq_delete(dds_WcharSeq* sequence);

/**
 * Get length of dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 *
 * @return length of dds_Wchar sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_WcharSeq_length(dds_WcharSeq* sequence);

/**
 * Get capacity of dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 *
 * @return capacity of dds_Wchar sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_WcharSeq_capacity(dds_WcharSeq* sequence);

/**
 * Add an item to dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_WcharSeq_add(dds_WcharSeq* sequence, dds_Wchar item);

/**
 * Add an item to dds_Wchar sequence at given position
 *
 * @param dds_Wchar sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_WcharSeq_add_at(dds_WcharSeq* sequence, uint32_t index, dds_Wchar item);

/**
 * Remove an item from dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Wchar dds_WcharSeq_remove(dds_WcharSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Wchar dds_WcharSeq_get(dds_WcharSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_WcharSeq_add_array(dds_WcharSeq* sequence, dds_Wchar* array, uint32_t size);

/**
 * Get an array of items from dds_Wchar sequence
 *
 * @param dds_Wchar sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_WcharSeq_get_array(dds_WcharSeq* sequence, dds_Wchar* array, uint32_t offset, uint32_t size);

// dds_Wstring Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Wstring sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Wstring sequence
 */
GURUMDDS_C_DLLAPI dds_WstringSeq* dds_WstringSeq_create(uint32_t capacity);

/**
 * Copy dds_Wstring sequence from source to sequence
 *
 * @param sequence destination sequence
 * @param source source sequence
 *
 * @return size for copied dds_Wstring sequence
 */
GURUMDDS_C_DLLAPI size_t dds_WstringSeq_copy(dds_WstringSeq* sequence, dds_WstringSeq* source);

/**
 * Clone dds_Wstring sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Wstring sequence
 */
GURUMDDS_C_DLLAPI dds_WstringSeq* dds_WstringSeq_clone(dds_WstringSeq* sequence);

/**
 * Delete dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 */
GURUMDDS_C_DLLAPI void dds_WstringSeq_delete(dds_WstringSeq* sequence);

/**
 * Get length of dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 *
 * @return length of dds_Wstring sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_WstringSeq_length(dds_WstringSeq* sequence);

/**
 * Get capacity of dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 *
 * @return capacity of dds_Wstring sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_WstringSeq_capacity(dds_WstringSeq* sequence);

/**
 * Add an item to dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_WstringSeq_add(dds_WstringSeq* sequence, dds_Wstring item);

/**
 * Add an item to dds_Wstring sequence at given position
 *
 * @param dds_Wstring sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_WstringSeq_add_at(dds_WstringSeq* sequence, uint32_t index, dds_Wstring item);

/**
 * Remove an item from dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Wstring dds_WstringSeq_remove(dds_WstringSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Wstring dds_WstringSeq_get(dds_WstringSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_WstringSeq_add_array(dds_WstringSeq* sequence, dds_Wstring* array, uint32_t size);

/**
 * Get an array of items from dds_Wstring sequence
 *
 * @param dds_Wstring sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_WstringSeq_get_array(dds_WstringSeq* sequence, dds_Wstring* array, uint32_t offset, uint32_t size);

// dds_Octet Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Octet sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Octet sequence
 */
GURUMDDS_C_DLLAPI dds_OctetSeq* dds_OctetSeq_create(uint32_t capacity);

/**
 * Clone dds_Octet sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Octet sequence
 */
GURUMDDS_C_DLLAPI dds_OctetSeq* dds_OctetSeq_clone(dds_OctetSeq* sequence);

/**
 * Delete dds_Octet sequence
 *
 * @param dds_Octet sequence
 */
GURUMDDS_C_DLLAPI void dds_OctetSeq_delete(dds_OctetSeq* sequence);

/**
 * Get length of dds_Octet sequence
 *
 * @param dds_Octet sequence
 *
 * @return length of dds_Octet sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_OctetSeq_length(dds_OctetSeq* sequence);

/**
 * Get capacity of dds_Octet sequence
 *
 * @param dds_Octet sequence
 *
 * @return capacity of dds_Octet sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_OctetSeq_capacity(dds_OctetSeq* sequence);

/**
 * Get array of dds_Octet sequence
 *
 * @param dds_Octet sequence
 *
 * @return array of dds_Octet sequence
 */
GURUMDDS_C_DLLAPI void* dds_OctetSeq_toarray(dds_OctetSeq* sequence);

/**
 * Add an item to dds_Octet sequence
 *
 * @param dds_Octet sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_OctetSeq_add(dds_OctetSeq* sequence, dds_Octet item);

/**
 * Add an item to dds_Octet sequence at given position
 *
 * @param dds_Octet sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_OctetSeq_add_at(dds_OctetSeq* sequence, uint32_t index, dds_Octet item);

/**
 * Remove an item from dds_Octet sequence
 *
 * @param dds_Octet sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Octet dds_OctetSeq_remove(dds_OctetSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Octet sequence
 *
 * @param dds_Octet sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Octet dds_OctetSeq_get(dds_OctetSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Octet sequence
 *
 * @param dds_Octet sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_OctetSeq_add_array(dds_OctetSeq* sequence, dds_Octet* array, uint32_t size);

/**
 * Get an array of items from dds_Octet sequence
 *
 * @param dds_Octet sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_OctetSeq_get_array(dds_OctetSeq* sequence, dds_Octet* array, uint32_t offset, uint32_t size);

// dds_Short Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Short sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Short sequence
 */
GURUMDDS_C_DLLAPI dds_ShortSeq* dds_ShortSeq_create(uint32_t capacity);

/**
 * Clone dds_Short sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Short sequence
 */
GURUMDDS_C_DLLAPI dds_ShortSeq* dds_ShortSeq_clone(dds_ShortSeq* sequence);

/**
 * Delete dds_Short sequence
 *
 * @param dds_Short sequence
 */
GURUMDDS_C_DLLAPI void dds_ShortSeq_delete(dds_ShortSeq* sequence);

/**
 * Get length of dds_Short sequence
 *
 * @param dds_Short sequence
 *
 * @return length of dds_Short sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_ShortSeq_length(dds_ShortSeq* sequence);

/**
 * Get capacity of dds_Short sequence
 *
 * @param dds_Short sequence
 *
 * @return capacity of dds_Short sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_ShortSeq_capacity(dds_ShortSeq* sequence);

/**
 * Add an item to dds_Short sequence
 *
 * @param dds_Short sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_ShortSeq_add(dds_ShortSeq* sequence, dds_Short item);

/**
 * Add an item to dds_Short sequence at given position
 *
 * @param dds_Short sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_ShortSeq_add_at(dds_ShortSeq* sequence, uint32_t index, dds_Short item);

/**
 * Remove an item from dds_Short sequence
 *
 * @param dds_Short sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Short dds_ShortSeq_remove(dds_ShortSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Short sequence
 *
 * @param dds_Short sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Short dds_ShortSeq_get(dds_ShortSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Short sequence
 *
 * @param dds_Short sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_ShortSeq_add_array(dds_ShortSeq* sequence, dds_Short* array, uint32_t size);

/**
 * Get an array of items from dds_Short sequence
 *
 * @param dds_Short sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_ShortSeq_get_array(dds_ShortSeq* sequence, dds_Short* array, uint32_t offset, uint32_t size);

// dds_UnsignedShort Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_UnsignedShort sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_UnsignedShort sequence
 */
GURUMDDS_C_DLLAPI dds_UnsignedShortSeq* dds_UnsignedShortSeq_create(uint32_t capacity);

/**
 * Clone dds_UnsignedShort sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_UnsignedShort sequence
 */
GURUMDDS_C_DLLAPI dds_UnsignedShortSeq* dds_UnsignedShortSeq_clone(dds_UnsignedShortSeq* sequence);

/**
 * Delete dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 */
GURUMDDS_C_DLLAPI void dds_UnsignedShortSeq_delete(dds_UnsignedShortSeq* sequence);

/**
 * Get length of dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 *
 * @return length of dds_UnsignedShort sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedShortSeq_length(dds_UnsignedShortSeq* sequence);

/**
 * Get capacity of dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 *
 * @return capacity of dds_UnsignedShort sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedShortSeq_capacity(dds_UnsignedShortSeq* sequence);

/**
 * Add an item to dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedShortSeq_add(dds_UnsignedShortSeq* sequence, dds_UnsignedShort item);

/**
 * Add an item to dds_UnsignedShort sequence at given position
 *
 * @param dds_UnsignedShort sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedShortSeq_add_at(dds_UnsignedShortSeq* sequence, uint32_t index, dds_UnsignedShort item);

/**
 * Remove an item from dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_UnsignedShort dds_UnsignedShortSeq_remove(dds_UnsignedShortSeq* sequence, uint32_t index);

/**
 * Get an item from dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_UnsignedShort dds_UnsignedShortSeq_get(dds_UnsignedShortSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedShortSeq_add_array(dds_UnsignedShortSeq* sequence, dds_UnsignedShort* array, uint32_t size);

/**
 * Get an array of items from dds_UnsignedShort sequence
 *
 * @param dds_UnsignedShort sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedShortSeq_get_array(dds_UnsignedShortSeq* sequence, dds_UnsignedShort* array, uint32_t offset, uint32_t size);

// dds_Long Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Long sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Long sequence
 */
GURUMDDS_C_DLLAPI dds_LongSeq* dds_LongSeq_create(uint32_t capacity);

/**
 * Clone dds_Long sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Long sequence
 */
GURUMDDS_C_DLLAPI dds_LongSeq* dds_LongSeq_clone(dds_LongSeq* sequence);

/**
 * Delete dds_Long sequence
 *
 * @param dds_Long sequence
 */
GURUMDDS_C_DLLAPI void dds_LongSeq_delete(dds_LongSeq* sequence);

/**
 * Get length of dds_Long sequence
 *
 * @param dds_Long sequence
 *
 * @return length of dds_Long sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_LongSeq_length(dds_LongSeq* sequence);

/**
 * Get capacity of dds_Long sequence
 *
 * @param dds_Long sequence
 *
 * @return capacity of dds_Long sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_LongSeq_capacity(dds_LongSeq* sequence);

/**
 * Add an item to dds_Long sequence
 *
 * @param dds_Long sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_LongSeq_add(dds_LongSeq* sequence, dds_Long item);

/**
 * Add an item to dds_Long sequence at given position
 *
 * @param dds_Long sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_LongSeq_add_at(dds_LongSeq* sequence, uint32_t index, dds_Long item);

/**
 * Remove an item from dds_Long sequence
 *
 * @param dds_Long sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Long dds_LongSeq_remove(dds_LongSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Long sequence
 *
 * @param dds_Long sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Long dds_LongSeq_get(dds_LongSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Long sequence
 *
 * @param dds_Long sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_LongSeq_add_array(dds_LongSeq* sequence, dds_Long* array, uint32_t size);

/**
 * Get an array of items from dds_Long sequence
 *
 * @param dds_Long sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_LongSeq_get_array(dds_LongSeq* sequence, dds_Long* array, uint32_t offset, uint32_t size);

// dds_UnsignedLong Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_UnsignedLong sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_UnsignedLong sequence
 */
GURUMDDS_C_DLLAPI dds_UnsignedLongSeq* dds_UnsignedLongSeq_create(uint32_t capacity);

/**
 * Clone dds_UnsignedLong sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_UnsignedLong sequence
 */
GURUMDDS_C_DLLAPI dds_UnsignedLongSeq* dds_UnsignedLongSeq_clone(dds_UnsignedLongSeq* sequence);

/**
 * Delete dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 */
GURUMDDS_C_DLLAPI void dds_UnsignedLongSeq_delete(dds_UnsignedLongSeq* sequence);

/**
 * Get length of dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 *
 * @return length of dds_UnsignedLong sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedLongSeq_length(dds_UnsignedLongSeq* sequence);

/**
 * Get capacity of dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 *
 * @return capacity of dds_UnsignedLong sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedLongSeq_capacity(dds_UnsignedLongSeq* sequence);

/**
 * Add an item to dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedLongSeq_add(dds_UnsignedLongSeq* sequence, dds_UnsignedLong item);

/**
 * Add an item to dds_UnsignedLong sequence at given position
 *
 * @param dds_UnsignedLong sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedLongSeq_add_at(dds_UnsignedLongSeq* sequence, uint32_t index, dds_UnsignedLong item);

/**
 * Remove an item from dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_UnsignedLong dds_UnsignedLongSeq_remove(dds_UnsignedLongSeq* sequence, uint32_t index);

/**
 * Get an item from dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_UnsignedLong dds_UnsignedLongSeq_get(dds_UnsignedLongSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedLongSeq_add_array(dds_UnsignedLongSeq* sequence, dds_UnsignedLong* array, uint32_t size);

/**
 * Get an array of items from dds_UnsignedLong sequence
 *
 * @param dds_UnsignedLong sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedLongSeq_get_array(dds_UnsignedLongSeq* sequence, dds_UnsignedLong* array, uint32_t offset, uint32_t size);

// dds_LongLong Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_LongLong sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_LongLong sequence
 */
GURUMDDS_C_DLLAPI dds_LongLongSeq* dds_LongLongSeq_create(uint32_t capacity);

/**
 * Clone dds_LongLong sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_LongLong sequence
 */
GURUMDDS_C_DLLAPI dds_LongLongSeq* dds_LongLongSeq_clone(dds_LongLongSeq* sequence);

/**
 * Delete dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 */
GURUMDDS_C_DLLAPI void dds_LongLongSeq_delete(dds_LongLongSeq* sequence);

/**
 * Get length of dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 *
 * @return length of dds_LongLong sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_LongLongSeq_length(dds_LongLongSeq* sequence);

/**
 * Get capacity of dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 *
 * @return capacity of dds_LongLong sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_LongLongSeq_capacity(dds_LongLongSeq* sequence);

/**
 * Add an item to dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_LongLongSeq_add(dds_LongLongSeq* sequence, dds_LongLong item);

/**
 * Add an item to dds_LongLong sequence at given position
 *
 * @param dds_LongLong sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_LongLongSeq_add_at(dds_LongLongSeq* sequence, uint32_t index, dds_LongLong item);

/**
 * Remove an item from dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_LongLong dds_LongLongSeq_remove(dds_LongLongSeq* sequence, uint32_t index);

/**
 * Get an item from dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_LongLong dds_LongLongSeq_get(dds_LongLongSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_LongLongSeq_add_array(dds_LongLongSeq* sequence, dds_LongLong* array, uint32_t size);

/**
 * Get an array of items from dds_LongLong sequence
 *
 * @param dds_LongLong sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_LongLongSeq_get_array(dds_LongLongSeq* sequence, dds_LongLong* array, uint32_t offset, uint32_t size);

// dds_UnsignedLongLong Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_UnsignedLongLong sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_UnsignedLongLong sequence
 */
GURUMDDS_C_DLLAPI dds_UnsignedLongLongSeq* dds_UnsignedLongLongSeq_create(uint32_t capacity);

/**
 * Clone dds_UnsignedLongLong sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_UnsignedLongLong sequence
 */
GURUMDDS_C_DLLAPI dds_UnsignedLongLongSeq* dds_UnsignedLongLongSeq_clone(dds_UnsignedLongLongSeq* sequence);

/**
 * Delete dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 */
GURUMDDS_C_DLLAPI void dds_UnsignedLongLongSeq_delete(dds_UnsignedLongLongSeq* sequence);

/**
 * Get length of dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 *
 * @return length of dds_UnsignedLongLong sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedLongLongSeq_length(dds_UnsignedLongLongSeq* sequence);

/**
 * Get capacity of dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 *
 * @return capacity of dds_UnsignedLongLong sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedLongLongSeq_capacity(dds_UnsignedLongLongSeq* sequence);

/**
 * Add an item to dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedLongLongSeq_add(dds_UnsignedLongLongSeq* sequence, dds_UnsignedLongLong item);

/**
 * Add an item to dds_UnsignedLongLong sequence at given position
 *
 * @param dds_UnsignedLongLong sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedLongLongSeq_add_at(dds_UnsignedLongLongSeq* sequence, uint32_t index, dds_UnsignedLongLong item);

/**
 * Remove an item from dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_UnsignedLongLong dds_UnsignedLongLongSeq_remove(dds_UnsignedLongLongSeq* sequence, uint32_t index);

/**
 * Get an item from dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_UnsignedLongLong dds_UnsignedLongLongSeq_get(dds_UnsignedLongLongSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_UnsignedLongLongSeq_add_array(dds_UnsignedLongLongSeq* sequence, dds_UnsignedLongLong* array, uint32_t size);

/**
 * Get an array of items from dds_UnsignedLongLong sequence
 *
 * @param dds_UnsignedLongLong sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_UnsignedLongLongSeq_get_array(dds_UnsignedLongLongSeq* sequence, dds_UnsignedLongLong* array, uint32_t offset, uint32_t size);

// dds_Float Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Float sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Float sequence
 */
GURUMDDS_C_DLLAPI dds_FloatSeq* dds_FloatSeq_create(uint32_t capacity);

/**
 * Clone dds_Float sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Float sequence
 */
GURUMDDS_C_DLLAPI dds_FloatSeq* dds_FloatSeq_clone(dds_FloatSeq* sequence);

/**
 * Delete dds_Float sequence
 *
 * @param dds_Float sequence
 */
GURUMDDS_C_DLLAPI void dds_FloatSeq_delete(dds_FloatSeq* sequence);

/**
 * Get length of dds_Float sequence
 *
 * @param dds_Float sequence
 *
 * @return length of dds_Float sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_FloatSeq_length(dds_FloatSeq* sequence);

/**
 * Get capacity of dds_Float sequence
 *
 * @param dds_Float sequence
 *
 * @return capacity of dds_Float sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_FloatSeq_capacity(dds_FloatSeq* sequence);

/**
 * Add an item to dds_Float sequence
 *
 * @param dds_Float sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_FloatSeq_add(dds_FloatSeq* sequence, dds_Float item);

/**
 * Add an item to dds_Float sequence at given position
 *
 * @param dds_Float sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_FloatSeq_add_at(dds_FloatSeq* sequence, uint32_t index, dds_Float item);

/**
 * Remove an item from dds_Float sequence
 *
 * @param dds_Float sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Float dds_FloatSeq_remove(dds_FloatSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Float sequence
 *
 * @param dds_Float sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Float dds_FloatSeq_get(dds_FloatSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Float sequence
 *
 * @param dds_Float sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_FloatSeq_add_array(dds_FloatSeq* sequence, dds_Float* array, uint32_t size);

/**
 * Get an array of items from dds_Float sequence
 *
 * @param dds_Float sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_FloatSeq_get_array(dds_FloatSeq* sequence, dds_Float* array, uint32_t offset, uint32_t size);

// dds_Double Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Double sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Double sequence
 */
GURUMDDS_C_DLLAPI dds_DoubleSeq* dds_DoubleSeq_create(uint32_t capacity);

/**
 * Clone dds_Double sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Double sequence
 */
GURUMDDS_C_DLLAPI dds_DoubleSeq* dds_DoubleSeq_clone(dds_DoubleSeq* sequence);

/**
 * Delete dds_Double sequence
 *
 * @param dds_Double sequence
 */
GURUMDDS_C_DLLAPI void dds_DoubleSeq_delete(dds_DoubleSeq* sequence);

/**
 * Get length of dds_Double sequence
 *
 * @param dds_Double sequence
 *
 * @return length of dds_Double sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_DoubleSeq_length(dds_DoubleSeq* sequence);

/**
 * Get capacity of dds_Double sequence
 *
 * @param dds_Double sequence
 *
 * @return capacity of dds_Double sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_DoubleSeq_capacity(dds_DoubleSeq* sequence);

/**
 * Add an item to dds_Double sequence
 *
 * @param dds_Double sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_DoubleSeq_add(dds_DoubleSeq* sequence, dds_Double item);

/**
 * Add an item to dds_Double sequence at given position
 *
 * @param dds_Double sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_DoubleSeq_add_at(dds_DoubleSeq* sequence, uint32_t index, dds_Double item);

/**
 * Remove an item from dds_Double sequence
 *
 * @param dds_Double sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Double dds_DoubleSeq_remove(dds_DoubleSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Double sequence
 *
 * @param dds_Double sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Double dds_DoubleSeq_get(dds_DoubleSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Double sequence
 *
 * @param dds_Double sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_DoubleSeq_add_array(dds_DoubleSeq* sequence, dds_Double* array, uint32_t size);

/**
 * Get an array of items from dds_Double sequence
 *
 * @param dds_Double sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_DoubleSeq_get_array(dds_DoubleSeq* sequence, dds_Double* array, uint32_t offset, uint32_t size);

// dds_Boolean Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Boolean sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Boolean sequence
 */
GURUMDDS_C_DLLAPI dds_BooleanSeq* dds_BooleanSeq_create(uint32_t capacity);

/**
 * Clone dds_Boolean sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Boolean sequence
 */
GURUMDDS_C_DLLAPI dds_BooleanSeq* dds_BooleanSeq_clone(dds_BooleanSeq* sequence);

/**
 * Delete dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 */
GURUMDDS_C_DLLAPI void dds_BooleanSeq_delete(dds_BooleanSeq* sequence);

/**
 * Get length of dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 *
 * @return length of dds_Boolean sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_BooleanSeq_length(dds_BooleanSeq* sequence);

/**
 * Get capacity of dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 *
 * @return capacity of dds_Boolean sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_BooleanSeq_capacity(dds_BooleanSeq* sequence);

/**
 * Add an item to dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_BooleanSeq_add(dds_BooleanSeq* sequence, dds_Boolean item);

/**
 * Add an item to dds_Boolean sequence at given position
 *
 * @param dds_Boolean sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_BooleanSeq_add_at(dds_BooleanSeq* sequence, uint32_t index, dds_Boolean item);

/**
 * Remove an item from dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Boolean dds_BooleanSeq_remove(dds_BooleanSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Boolean dds_BooleanSeq_get(dds_BooleanSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_BooleanSeq_add_array(dds_BooleanSeq* sequence, dds_Boolean* array, uint32_t size);

/**
 * Get an array of items from dds_Boolean sequence
 *
 * @param dds_Boolean sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_BooleanSeq_get_array(dds_BooleanSeq* sequence, dds_Boolean* array, uint32_t offset, uint32_t size);

// ----------------------------------------------------------------------
// Built-in Types
// ----------------------------------------------------------------------

typedef struct {
	dds_String key;
	dds_String value;
} dds_KeyedString;
typedef struct dds_KeyedStringSeq		dds_KeyedStringSeq;

typedef struct {
	dds_OctetSeq* value;
} dds_Bytes;
typedef struct dds_BytesSeq				dds_BytesSeq;

typedef struct {
	dds_String key;
	dds_OctetSeq* value;
} dds_KeyedBytes;
typedef struct dds_KeyedBytesSeq		dds_KeyedBytesSeq;

// dds_KeyedString Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_KeyedString sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI dds_KeyedStringSeq* dds_KeyedStringSeq_create(uint32_t capacity);

/**
 * Copy dds_KeyedString sequence from source to sequence
 *
 * @param sequence destination sequence
 * @param source source sequence
 *
 * @return size for copied dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI size_t dds_KeyedStringSeq_copy(dds_KeyedStringSeq* sequence, dds_KeyedStringSeq* source);

/**
 * Clone dds_KeyedString sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI dds_KeyedStringSeq* dds_KeyedStringSeq_clone(dds_KeyedStringSeq* sequence);

/**
 * Delete dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI void dds_KeyedStringSeq_delete(dds_KeyedStringSeq* sequence);

/**
 * Delete dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI void dds_KeyedStringSeq_delayed_delete(dds_KeyedStringSeq* sequence);

/**
 * Get length of dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 *
 * @return length of dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_KeyedStringSeq_length(dds_KeyedStringSeq* sequence);

/**
 * Get capacity of dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 *
 * @return capacity of dds_KeyedString sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_KeyedStringSeq_capacity(dds_KeyedStringSeq* sequence);

/**
 * Add an item to dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_KeyedStringSeq_add(dds_KeyedStringSeq* sequence, dds_KeyedString* item);

/**
 * Add an item to dds_KeyedString sequence at given position
 *
 * @param dds_KeyedString sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_KeyedStringSeq_add_at(dds_KeyedStringSeq* sequence, uint32_t index, dds_KeyedString* item);

/**
 * Remove an item from dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_KeyedString* dds_KeyedStringSeq_remove(dds_KeyedStringSeq* sequence, uint32_t index);

/**
 * Get an item from dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_KeyedString* dds_KeyedStringSeq_get(dds_KeyedStringSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_KeyedStringSeq_add_array(dds_KeyedStringSeq* sequence, dds_KeyedString** array, uint32_t size);

/**
 * Get an array of items from dds_KeyedString sequence
 *
 * @param dds_KeyedString sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_KeyedStringSeq_get_array(dds_KeyedStringSeq* sequence, dds_KeyedString** array, uint32_t offset, uint32_t size);

// dds_Bytes Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_Bytes sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI dds_BytesSeq* dds_BytesSeq_create(uint32_t capacity);

/**
 * Copy dds_Bytes sequence from source to sequence
 *
 * @param sequence destination sequence
 * @param source source sequence
 *
 * @return size for copied dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI size_t dds_BytesSeq_copy(dds_BytesSeq* sequence, dds_BytesSeq* source);

/**
 * Clone dds_Bytes sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI dds_BytesSeq* dds_BytesSeq_clone(dds_BytesSeq* sequence);

/**
 * Delete dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI void dds_BytesSeq_delete(dds_BytesSeq* sequence);

/**
 * Delete dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI void dds_BytesSeq_delayed_delete(dds_BytesSeq* sequence);

/**
 * Get length of dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 *
 * @return length of dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_BytesSeq_length(dds_BytesSeq* sequence);

/**
 * Get capacity of dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 *
 * @return capacity of dds_Bytes sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_BytesSeq_capacity(dds_BytesSeq* sequence);

/**
 * Add an item to dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_BytesSeq_add(dds_BytesSeq* sequence, dds_Bytes* item);

/**
 * Add an item to dds_Bytes sequence at given position
 *
 * @param dds_Bytes sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_BytesSeq_add_at(dds_BytesSeq* sequence, uint32_t index, dds_Bytes* item);

/**
 * Remove an item from dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_Bytes* dds_BytesSeq_remove(dds_BytesSeq* sequence, uint32_t index);

/**
 * Get an item from dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_Bytes* dds_BytesSeq_get(dds_BytesSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_BytesSeq_add_array(dds_BytesSeq* sequence, dds_Bytes** array, uint32_t size);

/**
 * Get an array of items from dds_Bytes sequence
 *
 * @param dds_Bytes sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_BytesSeq_get_array(dds_BytesSeq* sequence, dds_Bytes** array, uint32_t offset, uint32_t size);

// dds_KeyedBytes Sequence
// ---------------------------------------------------------------------
/**
 * Create dds_KeyedBytes sequence
 *
 * @param capacity initial capacity
 *
 * @return newly created dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI dds_KeyedBytesSeq* dds_KeyedBytesSeq_create(uint32_t capacity);

/**
 * Copy dds_KeyedBytes sequence from source to sequence
 *
 * @param sequence destination sequence
 * @param source source sequence
 *
 * @return size for copied dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI size_t dds_KeyedBytesSeq_copy(dds_KeyedBytesSeq* sequence, dds_KeyedBytesSeq* source);

/**
 * Clone dds_KeyedBytes sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI dds_KeyedBytesSeq* dds_KeyedBytesSeq_clone(dds_KeyedBytesSeq* sequence);

/**
 * Delete dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI void dds_KeyedBytesSeq_delete(dds_KeyedBytesSeq* sequence);

/**
 * Delete dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI void dds_KeyedBytesSeq_delayed_delete(dds_KeyedBytesSeq* sequence);

/**
 * Get length of dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 *
 * @return length of dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_KeyedBytesSeq_length(dds_KeyedBytesSeq* sequence);

/**
 * Get capacity of dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 *
 * @return capacity of dds_KeyedBytes sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_KeyedBytesSeq_capacity(dds_KeyedBytesSeq* sequence);

/**
 * Add an item to dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_KeyedBytesSeq_add(dds_KeyedBytesSeq* sequence, dds_KeyedBytes* item);

/**
 * Add an item to dds_KeyedBytes sequence at given position
 *
 * @param dds_KeyedBytes sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_KeyedBytesSeq_add_at(dds_KeyedBytesSeq* sequence, uint32_t index, dds_KeyedBytes* item);

/**
 * Remove an item from dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_KeyedBytes* dds_KeyedBytesSeq_remove(dds_KeyedBytesSeq* sequence, uint32_t index);

/**
 * Get an item from dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_KeyedBytes* dds_KeyedBytesSeq_get(dds_KeyedBytesSeq* sequence, uint32_t index);

/**
 * Add an array of items to dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_KeyedBytesSeq_add_array(dds_KeyedBytesSeq* sequence, dds_KeyedBytes** array, uint32_t size);

/**
 * Get an array of items from dds_KeyedBytes sequence
 *
 * @param dds_KeyedBytes sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_KeyedBytesSeq_get_array(dds_KeyedBytesSeq* sequence, dds_KeyedBytes** array, uint32_t offset, uint32_t size);

// dds_String TypeSupport
// ---------------------------------------------------------------------

/** @see dds_TypeSupport_register_type */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringTypeSupport_register_type(const dds_DomainParticipant* domain, const char* type_name);
/** @see dds_TypeSupport_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_StringTypeSupport_get_type_name();
/** @see dds_TypeSupport_serialize */
GURUMDDS_C_DLLAPI void* dds_StringTypeSupport_serialize(dds_String data, size_t* output_size);
/** @see dds_TypeSupport_deserialize */
GURUMDDS_C_DLLAPI dds_String dds_StringTypeSupport_deserialize(void* input, size_t input_size);

/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataWriter_write(dds_DataWriter* self, const dds_String instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataWriter_write_w_timestamp(dds_DataWriter* self, const dds_String instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);

/** @see dds_DataReader_read */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_read(dds_DataReader* self, dds_StringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_take(dds_DataReader* self, dds_StringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_read_w_condition(dds_DataReader* self, dds_StringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_take_w_condition(dds_DataReader* self, dds_StringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_read_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_read_next_sample(dds_DataReader* self, dds_String data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_take_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_take_next_sample(dds_DataReader* self, dds_String data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_return_loan */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StringDataReader_return_loan(dds_DataReader* self, dds_StringSeq* data_values, dds_SampleInfoSeq* sample_infos);

// dds_KeyedString TypeSupport
// ---------------------------------------------------------------------

/** @see dds_TypeSupport_register_type */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringTypeSupport_register_type(const dds_DomainParticipant* domain, const char* type_name);
/** @see dds_TypeSupport_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_KeyedStringTypeSupport_get_type_name();
/** @see dds_TypeSupport_alloc */
GURUMDDS_C_DLLAPI dds_KeyedString* dds_KeyedStringTypeSupport_alloc();
/** @see dds_TypeSupport_free */
GURUMDDS_C_DLLAPI void dds_KeyedStringTypeSupport_free(dds_KeyedString* msg);
/** @see dds_TypeSupport_serialize */
GURUMDDS_C_DLLAPI void* dds_KeyedStringTypeSupport_serialize(dds_KeyedString* data, size_t* output_size);
/** @see dds_TypeSupport_deserialize */
GURUMDDS_C_DLLAPI dds_KeyedString* dds_KeyedStringTypeSupport_deserialize(void* input, size_t input_size);

/** @see dds_DataWriter_register_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataWriter_register_instance(dds_DataWriter* self, const dds_KeyedString* instance_data);
/** @see dds_DataWriter_register_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataWriter_register_instance_w_timestamp(dds_DataWriter* self, const dds_KeyedString* instance_data, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_unregister_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_unregister_instance(dds_DataWriter* self, const dds_KeyedString* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_unregister_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_unregister_instance_w_timestamp(dds_DataWriter* self, const dds_KeyedString* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_write(dds_DataWriter* self, const dds_KeyedString* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_write_w_timestamp(dds_DataWriter* self, const dds_KeyedString* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_dispose */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_dispose(dds_DataWriter* self, const dds_KeyedString* instance_data, dds_InstanceHandle_t instance_handle);
/** @see dds_DataWriter_dispose_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_dispose_w_timestamp(dds_DataWriter* self, const dds_KeyedString* instance_data, dds_InstanceHandle_t instance_handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_get_key_value(dds_DataWriter* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataWriter_lookup_instance(dds_DataWriter* self, const dds_KeyedString* instance_data);

/** @see dds_DataReader_read */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_read(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_take(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_read_w_condition(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_take_w_condition(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_read_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_read_next_sample(dds_DataReader* self, dds_KeyedString* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_take_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_take_next_sample(dds_DataReader* self, dds_KeyedString* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_read_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_read_instance(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_take_instance(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_read_next_instance(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_take_next_instance(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_read_next_instance_w_condition(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_take_next_instance_w_condition(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_return_loan */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_return_loan(dds_DataReader* self, dds_KeyedStringSeq* data_values, dds_SampleInfoSeq* sample_infos);
/** @see dds_DataReader_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataReader_get_key_value(dds_DataReader* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataReader_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataReader_lookup_instance(dds_DataReader* self, const dds_KeyedString* instance_data);

/** @see dds_DataWriter_register_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataWriter_register_instance_w_key(dds_DataWriter* self, const dds_String key);
/** @see dds_DataWriter_register_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataWriter_register_instance_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_unregister_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_unregister_instance_w_key(dds_DataWriter* self, const dds_String key);
/** @see dds_DataWriter_unregister_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_unregister_instance_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_write_string_w_key(dds_DataWriter* self, const dds_String key, const dds_String str, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_write_string_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_String str, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_dispose */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_dispose_w_key(dds_DataWriter* self, const dds_String key);
/** @see dds_DataWriter_dispose_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_dispose_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedStringDataWriter_get_key_value_w_key(dds_DataWriter* self, dds_String key, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedStringDataWriter_lookup_instance_w_key(dds_DataWriter* self, const dds_String instance_data);

// dds_Bytes TypeSupport
// ---------------------------------------------------------------------

/** @see dds_TypeSupport_register_type */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesTypeSupport_register_type(const dds_DomainParticipant* domain, const char* type_name);
/** @see dds_TypeSupport_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_BytesTypeSupport_get_type_name();
/** @see dds_TypeSupport_alloc */
GURUMDDS_C_DLLAPI dds_Bytes* dds_BytesTypeSupport_alloc();
/** @see dds_TypeSupport_free */
GURUMDDS_C_DLLAPI void dds_BytesTypeSupport_free(dds_Bytes* msg);
/** @see dds_TypeSupport_serialize */
GURUMDDS_C_DLLAPI void* dds_BytesTypeSupport_serialize(dds_Bytes* data, size_t* output_size);
/** @see dds_TypeSupport_deserialize */
GURUMDDS_C_DLLAPI dds_Bytes* dds_BytesTypeSupport_deserialize(void* input, size_t input_size);

/** @see dds_DataWriter_register_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_BytesDataWriter_register_instance(dds_DataWriter* self, const dds_Bytes* instance_data);
/** @see dds_DataWriter_register_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_BytesDataWriter_register_instance_w_timestamp(dds_DataWriter* self, const dds_Bytes* instance_data, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_unregister_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_unregister_instance(dds_DataWriter* self, const dds_Bytes* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_unregister_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_unregister_instance_w_timestamp(dds_DataWriter* self, const dds_Bytes* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_write(dds_DataWriter* self, const dds_Bytes* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_write_w_timestamp(dds_DataWriter* self, const dds_Bytes* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_dispose */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_dispose(dds_DataWriter* self, const dds_Bytes* instance_data, dds_InstanceHandle_t instance_handle);
/** @see dds_DataWriter_dispose_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_dispose_w_timestamp(dds_DataWriter* self, const dds_Bytes* instance_data, dds_InstanceHandle_t instance_handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_get_key_value(dds_DataWriter* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_BytesDataWriter_lookup_instance(dds_DataWriter* self, const dds_Bytes* instance_data);

/** @see dds_DataReader_read */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_read(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_take(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_read_w_condition(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_take_w_condition(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_read_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_read_next_sample(dds_DataReader* self, dds_Bytes* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_take_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_take_next_sample(dds_DataReader* self, dds_Bytes* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_read_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_read_instance(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_take_instance(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_read_next_instance(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_take_next_instance(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_read_next_instance_w_condition(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_take_next_instance_w_condition(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_return_loan */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_return_loan(dds_DataReader* self, dds_BytesSeq* data_values, dds_SampleInfoSeq* sample_infos);
/** @see dds_DataReader_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataReader_get_key_value(dds_DataReader* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataReader_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_BytesDataReader_lookup_instance(dds_DataReader* self, const dds_Bytes* instance_data);

/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_write_w_bytes(dds_DataWriter* self, const dds_Octet* bytes, dds_Long offset, dds_Long length, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_BytesDataWriter_write_w_bytes_w_timestamp(dds_DataWriter* self, const dds_Octet* bytes, dds_Long offset, dds_Long length, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);

// dds_KeyedBytes TypeSupport
// ---------------------------------------------------------------------

/** @see dds_TypeSupport_register_type */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesTypeSupport_register_type(const dds_DomainParticipant* domain, const char* type_name);
/** @see dds_TypeSupport_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_KeyedBytesTypeSupport_get_type_name();
/** @see dds_TypeSupport_alloc */
GURUMDDS_C_DLLAPI dds_KeyedBytes* dds_KeyedBytesTypeSupport_alloc();
/** @see dds_TypeSupport_free */
GURUMDDS_C_DLLAPI void dds_KeyedBytesTypeSupport_free(dds_KeyedBytes* msg);
/** @see dds_TypeSupport_serialize */
GURUMDDS_C_DLLAPI void* dds_KeyedBytesTypeSupport_serialize(dds_KeyedBytes* data, size_t* output_size);
/** @see dds_TypeSupport_deserialize */
GURUMDDS_C_DLLAPI dds_KeyedBytes* dds_KeyedBytesTypeSupport_deserialize(void* input, size_t input_size);

/** @see dds_DataWriter_register_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_register_instance(dds_DataWriter* self, const dds_KeyedBytes* instance_data);
/** @see dds_DataWriter_register_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_register_instance_w_timestamp(dds_DataWriter* self, const dds_KeyedBytes* instance_data, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_unregister_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_unregister_instance(dds_DataWriter* self, const dds_KeyedBytes* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_unregister_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_unregister_instance_w_timestamp(dds_DataWriter* self, const dds_KeyedBytes* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_write(dds_DataWriter* self, const dds_KeyedBytes* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_write_w_timestamp(dds_DataWriter* self, const dds_KeyedBytes* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_dispose */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_dispose(dds_DataWriter* self, const dds_KeyedBytes* instance_data, dds_InstanceHandle_t instance_handle);
/** @see dds_DataWriter_dispose_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_dispose_w_timestamp(dds_DataWriter* self, const dds_KeyedBytes* instance_data, dds_InstanceHandle_t instance_handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_get_key_value(dds_DataWriter* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_lookup_instance(dds_DataWriter* self, const dds_KeyedBytes* instance_data);

/** @see dds_DataReader_read */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_read(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_take(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_read_w_condition(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_take_w_condition(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_read_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_read_next_sample(dds_DataReader* self, dds_KeyedBytes* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_take_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_take_next_sample(dds_DataReader* self, dds_KeyedBytes* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_read_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_read_instance(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_take_instance(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_read_next_instance(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_take_next_instance(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_read_next_instance_w_condition(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_take_next_instance_w_condition(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_return_loan */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_return_loan(dds_DataReader* self, dds_KeyedBytesSeq* data_values, dds_SampleInfoSeq* sample_infos);
/** @see dds_DataReader_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataReader_get_key_value(dds_DataReader* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataReader_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataReader_lookup_instance(dds_DataReader* self, const dds_KeyedBytes* instance_data);

/** @see dds_DataWriter_register_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_register_instance_w_key(dds_DataWriter* self, const dds_String key);
/** @see dds_DataWriter_register_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_register_instance_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_unregister_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_unregister_instance_w_key(dds_DataWriter* self, const dds_String key);
/** @see dds_DataWriter_unregister_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_unregister_instance_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_write_bytes_w_key(dds_DataWriter* self, const dds_String key, const dds_Octet* bytes, dds_Long offset, dds_Long length, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_write_bytes_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Octet* bytes, dds_Long offset, dds_Long length, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_dispose */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_dispose_w_key(dds_DataWriter* self, const dds_String key);
/** @see dds_DataWriter_dispose_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_dispose_w_key_w_timestamp(dds_DataWriter* self, const dds_String key, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_KeyedBytesDataWriter_get_key_value_w_key(dds_DataWriter* self, dds_String key, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_lookup-instance_w_key */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_KeyedBytesDataWriter_lookup_instance_w_key(dds_DataWriter *self, const dds_String key);

#ifdef __cplusplus
}
#endif

#endif /* __GURUMDDS_TYPESUPPORT_H__ */

